//---------------------------------------------------------------------
// Authors: jachymko
//
// Description: Base class for tab expansion handlers.
//
// Creation Date: Dec 27, 2006
//
//---------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Management.Automation;
using System.Management.Automation.Runspaces;
using BGShell.PowerShell.Commands;

namespace BGShell.PowerShell.TabExpansion
{
    internal abstract class TabExpansionHandler
    {
        private GetTabExpansionCommand _command;

        protected TabExpansionHandler(GetTabExpansionCommand command)
        {
            _command = command;
        }

        /// <returns>true if the remaining handlers are to be examined, false otherwise</returns>
        public abstract bool TryExpand(String line, String word);

        protected GetTabExpansionCommand Command
        {
            get { return _command; }
        }

        protected void WriteExpansions(params string[] expansions)
        {
            _command.WriteExpansions(expansions);
        }

        protected void WriteExpansions(IEnumerable<string> expansions)
        {
            _command.WriteExpansions(expansions);
        }

        protected Collection<PSObject> Invoke(string script, object[] input, object[] args)
        {
            return Command.InvokeCommand.InvokeScript(
                script, false, 
                PipelineResultTypes.None,
                input, args);
        }

        protected Collection<PSObject> InvokeArgs(string script, params object[] args)
        {
            return Invoke(script, null, args);
        }

        //protected Collection<PSObject> InvokeInput(string script, object[] input)
        //{
        //    return Invoke(script, input, null);
        //}
        
        protected Collection<PSObject> Invoke(string script)
        {
            return Invoke(script, null, null);
        }

        protected PSObject InvokeReturnOne(string script, params object[] args)
        {
            Collection<PSObject> results = InvokeArgs(script, args);

            if(results.Count > 0)
            {
                return results[0];
            }

            return null;
        }

        protected CommandInfo ResolveAlias(string str)
        {
            PSObject pscmd = InvokeReturnOne("Microsoft.PowerShell.Core\\Get-Command -Type Filter,Function,Cmdlet,Alias $args[0]", str);

            if (pscmd == null)
            {
                return null;
            }

            AliasInfo alias = pscmd.BaseObject as AliasInfo;
            CommandInfo command = pscmd.BaseObject as CommandInfo;

            if (alias != null)
            {
                command = alias.ResolvedCommand as CommandInfo;
            }

            return command;
        }

        protected static bool IsMethod(PSMemberTypes member)
        {
            return 0 != (member & (PSMemberTypes.Methods | PSMemberTypes.ParameterizedProperty));
        }
    }
} 
